@@ -3,6 +3,9 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Aggregate top level objects from all Scapy modules.
+"""
 
 from base_classes import *
 from config import *
@@ -20,8 +23,9 @@ from asn1packet import *
 
 from utils import *
 from route import *
-from utils6 import *
-from route6 import *
+if conf.ipv6_enabled:
+    from utils6 import *
+    from route6 import *
 from sendrecv import *
 from supersocket import *
 from volatile import *
@@ -39,5 +43,4 @@ from asn1.asn1 import *
 from asn1.ber import *
 from asn1.mib import *
 
-
-
+from crypto import *
