@@ -3,16 +3,21 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Linux specific functions.
+"""
 
 from __future__ import with_statement
 import sys,os,struct,socket,time
 from select import select
 from fcntl import ioctl
 import scapy.utils
+import scapy.utils6
 from scapy.config import conf
 from scapy.data import *
 from scapy.supersocket import SuperSocket
 import scapy.arch
+from scapy.error import warning
 
 
 
@@ -64,13 +69,13 @@ RTF_REJECT = 0x0200
 
 LOOPBACK_NAME="lo"
 
-with os.popen("tcpdump -V 2> /dev/null") as f:
-    if f.close() >> 8 == 0x7f:
+with os.popen("tcpdump -V 2> /dev/null") as _f:
+    if _f.close() >> 8 == 0x7f:
         log_loading.warning("Failed to execute tcpdump. Check it is installed and in the PATH")
         TCPDUMP=0
     else:
         TCPDUMP=1
-        
+del(_f)
     
 
 def get_if_raw_hwaddr(iff):
@@ -180,6 +185,85 @@ def read_routes():
     f.close()
     return routes
 
+############
+### IPv6 ###
+############
+
+def in6_getifaddr():
+    """
+    Returns a list of 3-tuples of the form (addr, scope, iface) where
+    'addr' is the address of scope 'scope' associated to the interface
+    'ifcace'.
+
+    This is the list of all addresses of all interfaces available on
+    the system.
+    """
+    ret = []
+    try:
+        f = open("/proc/net/if_inet6","r")
+    except IOError, err:    
+        return ret
+    l = f.readlines()
+    for i in l:
+        # addr, index, plen, scope, flags, ifname
+        tmp = i.split()
+        addr = struct.unpack('4s4s4s4s4s4s4s4s', tmp[0])
+        addr = scapy.utils6.in6_ptop(':'.join(addr))
+        ret.append((addr, int(tmp[3], 16), tmp[5])) # (addr, scope, iface)
+    return ret
+
+def read_routes6():
+    try:
+        f = open("/proc/net/ipv6_route","r")
+    except IOError, err:
+        return []
+    # 1. destination network
+    # 2. destination prefix length
+    # 3. source network displayed
+    # 4. source prefix length
+    # 5. next hop
+    # 6. metric
+    # 7. reference counter (?!?)
+    # 8. use counter (?!?)
+    # 9. flags
+    # 10. device name
+    routes = []
+    def proc2r(p):
+        ret = struct.unpack('4s4s4s4s4s4s4s4s', p)
+        ret = ':'.join(ret)
+        return scapy.utils6.in6_ptop(ret)
+    
+    lifaddr = in6_getifaddr() 
+    for l in f.readlines():
+        d,dp,s,sp,nh,m,rc,us,fl,dev = l.split()
+        fl = int(fl, 16)
+
+        if fl & RTF_UP == 0:
+            continue
+        if fl & RTF_REJECT:
+            continue
+
+        d = proc2r(d) ; dp = int(dp, 16)
+        s = proc2r(s) ; sp = int(sp, 16)
+        nh = proc2r(nh)
+
+        cset = [] # candidate set (possible source addresses)
+        if dev == LOOPBACK_NAME:
+            if d == '::':
+                continue
+            cset = ['::1']
+        else:
+            devaddrs = filter(lambda x: x[2] == dev, lifaddr)
+            cset = scapy.utils6.construct_source_candidate_set(d, dp, devaddrs, LOOPBACK_NAME)
+        
+        if len(cset) != 0:
+            routes.append((d, dp, nh, dev, cset))
+    f.close()
+    return routes   
+
+
+
+
 def get_if(iff,cmd):
     s=socket.socket()
     ifreq = ioctl(s, cmd, struct.pack("16s16x",iff))
@@ -190,11 +274,16 @@ def get_if(iff,cmd):
 def get_if_index(iff):
     return int(struct.unpack("I",get_if(iff, SIOCGIFINDEX)[16:20])[0])
 
-def get_last_packet_timestamp(sock):
-    ts = ioctl(sock, SIOCGSTAMP, "12345678")
-    s,us = struct.unpack("II",ts)
-    return s+us/1000000.0
-
+if os.uname()[4] == 'x86_64':
+    def get_last_packet_timestamp(sock):
+        ts = ioctl(sock, SIOCGSTAMP, "1234567890123456")
+        s,us = struct.unpack("QQ",ts)
+        return s+us/1000000.0
+else:
+    def get_last_packet_timestamp(sock):
+        ts = ioctl(sock, SIOCGSTAMP, "12345678")
+        s,us = struct.unpack("II",ts)
+        return s+us/1000000.0
 
 
 def _flush_fd(fd):
@@ -252,7 +341,7 @@ class L3PacketSocket(SuperSocket):
             for i in self.iff:
                 set_promisc(self.ins, i, 0)
         SuperSocket.close(self)
-    def recv(self, x):
+    def recv(self, x=MTU):
         pkt, sa_ll = self.ins.recvfrom(x)
         if sa_ll[2] == socket.PACKET_OUTGOING:
             return None
@@ -290,8 +379,8 @@ class L3PacketSocket(SuperSocket):
         self.outs.bind(sdto)
         sn = self.outs.getsockname()
         ll = lambda x:x
-        if sn[3] in (ARPHDR_PPP,ARPHDR_TUN):
-            sdto = (iff, ETH_P_IP)
+        if type(x) in conf.l3types:
+            sdto = (iff, conf.l3types[type(x)])
         if sn[3] in conf.l2types:
             ll = lambda x:conf.l2types[sn[3]]()/x
         try:
@@ -301,7 +390,7 @@ class L3PacketSocket(SuperSocket):
         except socket.error,msg:
             x.sent_time = time.time()  # bad approximation
             if conf.auto_fragment and msg[0] == 90:
-                for p in fragment(x):
+                for p in x.fragment():
                     self.outs.sendto(str(ll(p)), sdto)
             else:
                 raise
@@ -338,7 +427,7 @@ class L2Socket(SuperSocket):
             self.LL = conf.default_l2
             warning("Unable to guess type (interface=%s protocol=%#x family=%i). Using %s" % (sa_ll[0],sa_ll[1],sa_ll[3],self.LL.name))
             
-    def recv(self, x):
+    def recv(self, x=MTU):
         pkt, sa_ll = self.ins.recvfrom(x)
         if sa_ll[2] == socket.PACKET_OUTGOING:
             return None
