@@ -3,12 +3,18 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
-import os
+"""
+PacketList: holds several packets and allows to do operations on them.
+"""
+
+
+import os,subprocess
 from config import conf
 from base_classes import BasePacket,BasePacketList
 from packet import Padding
+from collections import defaultdict
 
-from utils import do_graph,hexdump,make_table,make_lined_table,make_tex_table
+from utils import do_graph,hexdump,make_table,make_lined_table,make_tex_table,get_temp_file
 
 import arch
 if arch.GNUPLOT:
@@ -104,7 +110,7 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
             if lfilter is not None:
                 if not lfilter(self.res[i]):
                     continue
-            print conf.color_theme.id(i,"%04i"),
+            print conf.color_theme.id(i,fmt="%04i"),
             if prn is None:
                 print self._elt2sum(self.res[i])
             else:
@@ -187,7 +193,7 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
             p = self._elt2pkt(self.res[i])
             if lfilter is not None and not lfilter(p):
                 continue
-            print "%s %s %s" % (conf.color_theme.id(i,"%04i"),
+            print "%s %s %s" % (conf.color_theme.id(i,fmt="%04i"),
                                 p.sprintf("%.time%"),
                                 self._elt2sum(self.res[i]))
             if p.haslayer(conf.raw_layer):
@@ -200,7 +206,7 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
             p = self._elt2pkt(self.res[i])
             if lfilter is not None and not lfilter(p):
                 continue
-            print "%s %s %s" % (conf.color_theme.id(i,"%04i"),
+            print "%s %s %s" % (conf.color_theme.id(i,fmt="%04i"),
                                 p.sprintf("%.time%"),
                                 self._elt2sum(self.res[i]))
             hexdump(p)
@@ -211,7 +217,7 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
             p = self._elt2pkt(self.res[i])
             if p.haslayer(Padding):
                 if lfilter is None or lfilter(p):
-                    print "%s %s %s" % (conf.color_theme.id(i,"%04i"),
+                    print "%s %s %s" % (conf.color_theme.id(i,fmt="%04i"),
                                         p.sprintf("%.time%"),
                                         self._elt2sum(self.res[i]))
                     hexdump(p.getlayer(Padding).load)
@@ -225,7 +231,7 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
                 if pad == pad[0]*len(pad):
                     continue
                 if lfilter is None or lfilter(p):
-                    print "%s %s %s" % (conf.color_theme.id(i,"%04i"),
+                    print "%s %s %s" % (conf.color_theme.id(i,fmt="%04i"),
                                         p.sprintf("%.time%"),
                                         self._elt2sum(self.res[i]))
                     hexdump(p.getlayer(Padding).load)
@@ -333,7 +339,6 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
                 gr += ' "evt.%s" -> "dst.%s";\n' % (`e`,`d`) 
             
         gr += "}"
-        open("/tmp/aze","w").write(gr)
         return do_graph(gr, **kargs)
 
 
@@ -361,9 +366,9 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
                   conf.prog.psreader is called"""
         d = self._dump_document(**kargs)
         if filename is None:
-            filename = "/tmp/scapy.psd.%i" % os.getpid()
+            filename = get_temp_file(autoext=".ps")
             d.writePSfile(filename)
-            os.system("%s %s.ps &" % (conf.prog.psreader,filename))
+            subprocess.Popen([conf.prog.psreader, filename+".ps"])
         else:
             d.writePSfile(filename)
         print
@@ -374,9 +379,9 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
                   conf.prog.pdfreader is called"""
         d = self._dump_document(**kargs)
         if filename is None:
-            filename = "/tmp/scapy.psd.%i" % os.getpid()
+            filename = get_temp_file(autoext=".pdf")
             d.writePDFfile(filename)
-            os.system("%s %s.pdf &" % (conf.prog.pdfreader,filename))
+            subprocess.Popen([conf.prog.pdfreader, filename+".pdf"])
         else:
             d.writePDFfile(filename)
         print
@@ -407,6 +412,68 @@ lfilter: truth function to apply to each packet to decide whether it will be dis
         if multi:
             remain = filter(lambda x:not hasattr(x,"_answered"), remain)
         return SndRcvList(sr),PacketList(remain)
+
+    def sessions(self, session_extractor=None):
+        if session_extractor is None:
+            def session_extractor(p):
+                sess = "Other"
+                if 'Ether' in p:
+                    if 'IP' in p:
+                        if 'TCP' in p:
+                            sess = p.sprintf("TCP %IP.src%:%r,TCP.sport% > %IP.dst%:%r,TCP.dport%")
+                        elif 'UDP' in p:
+                            sess = p.sprintf("UDP %IP.src%:%r,UDP.sport% > %IP.dst%:%r,UDP.dport%")
+                        elif 'ICMP' in p:
+                            sess = p.sprintf("ICMP %IP.src% > %IP.dst% type=%r,ICMP.type% code=%r,ICMP.code% id=%ICMP.id%")
+                        else:
+                            sess = p.sprintf("IP %IP.src% > %IP.dst% proto=%IP.proto%")
+                    elif 'ARP' in p:
+                        sess = p.sprintf("ARP %ARP.psrc% > %ARP.pdst%")
+                    else:
+                        sess = p.sprintf("Ethernet type=%04xr,Ether.type%")
+                return sess
+        sessions = defaultdict(self.__class__)
+        for p in self.res:
+            sess = session_extractor(self._elt2pkt(p))
+            sessions[sess].append(p)
+        return dict(sessions)
+    
+    def replace(self, *args, **kargs):
+        """
+        lst.replace(<field>,[<oldvalue>,]<newvalue>)
+        lst.replace( (fld,[ov],nv),(fld,[ov,]nv),...)
+          if ov is None, all values are replaced
+        ex:
+          lst.replace( IP.src, "192.168.1.1", "10.0.0.1" )
+          lst.replace( IP.ttl, 64 )
+          lst.replace( (IP.ttl, 64), (TCP.sport, 666, 777), )
+        """
+        delete_checksums = kargs.get("delete_checksums",False)
+        x=PacketList(name="Replaced %s" % self.listname)
+        if type(args[0]) is not tuple:
+            args = (args,)
+        for p in self.res:
+            p = self._elt2pkt(p)
+            copied = False
+            for scheme in args:
+                fld = scheme[0]
+                old = scheme[1] # not used if len(scheme) == 2
+                new = scheme[-1]
+                for o in fld.owners:
+                    if o in p:
+                        if len(scheme) == 2 or p[o].getfieldval(fld.name) == old:
+                            if not copied:
+                                p = p.copy()
+                                if delete_checksums:
+                                    p.delete_checksums()
+                                copied = True
+                            setattr(p[o], fld.name, new)
+            x.append(p)
+        return x
+                
+            
+        
+    
         
 
 
