@@ -3,6 +3,10 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Functions to send and receive packets.
+"""
+
 import cPickle,os,sys,time,subprocess
 from select import select
 from data import *
@@ -255,12 +259,13 @@ sendp(packets, [inter=0], [loop=0], [verbose=conf.verb]) -> None"""
     __gen_send(conf.L2socket(iface=iface, *args, **kargs), x, inter=inter, loop=loop, count=count, verbose=verbose, realtime=realtime)
 
 @conf.commands.register
-def sendpfast(x, pps=None, mbps=None, realtime=None, loop=0, iface=None):
+def sendpfast(x, pps=None, mbps=None, realtime=None, loop=0, file_cache=False, iface=None):
     """Send packets at layer 2 using tcpreplay for performance
     pps:  packets per second
     mpbs: MBits per second
     realtime: use packet's timestamp, bending time with realtime value
     loop: number of times to process the packet list
+    file_cache: cache packets in RAM instead of reading from disk at each iteration
     iface: output interface """
     if iface is None:
         iface = conf.iface
@@ -276,6 +281,8 @@ def sendpfast(x, pps=None, mbps=None, realtime=None, loop=0, iface=None):
 
     if loop:
         argv.append("--loop=%i" % loop)
+        if file_cache:
+            argv.append("--enable-file-cache")
 
     f = get_temp_file()
     argv.append(f)
@@ -522,7 +529,8 @@ iface:    listen answers only on the given interface"""
 
 
 @conf.commands.register
-def sniff(count=0, store=1, offline=None, prn = None, lfilter=None, L2socket=None, timeout=None, opened_socket=None, *arg, **karg):
+def sniff(count=0, store=1, offline=None, prn = None, lfilter=None, L2socket=None, timeout=None,
+          opened_socket=None, stop_filter=None, *arg, **karg):
     """Sniff packets
 sniff([count=0,] [prn=None,] [store=1,] [offline=None,] [lfilter=None,] + L2ListenSocket args) -> list of packets
 
@@ -538,6 +546,9 @@ offline: pcap file to read packets from, instead of sniffing them
 timeout: stop sniffing after a given time (default: None)
 L2socket: use the provided L2socket
 opened_socket: provide an object ready to use .recv() on
+stop_filter: python function applied to each packet to determine
+             if we have to stop the capture after this packet
+             ex: stop_filter = lambda x: x.haslayer(TCP)
     """
     c = 0
     
@@ -575,6 +586,8 @@ opened_socket: provide an object ready to use .recv() on
                     r = prn(p)
                     if r is not None:
                         print r
+                if stop_filter and stop_filter(p):
+                    break
                 if count > 0 and c >= count:
                     break
         except KeyboardInterrupt:
