@@ -3,9 +3,14 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+SuperSocket.
+"""
+
 import socket,time
 from config import conf
 from data import *
+from scapy.error import warning
 
 class _SuperSocket_metaclass(type):
     def __repr__(self):
@@ -27,7 +32,7 @@ class SuperSocket:
         sx = str(x)
         x.sent_time = time.time()
         return self.outs.send(sx)
-    def recv(self, x):
+    def recv(self, x=MTU):
         return conf.raw_layer(self.ins.recv(x))
     def fileno(self):
         return self.ins.fileno()
@@ -40,10 +45,16 @@ class SuperSocket:
                 self.outs.close()
         if self.ins and self.ins.fileno() != -1:
             self.ins.close()
-    def bind_in(self, addr):
-        self.ins.bind(addr)
-    def bind_out(self, addr):
-        self.outs.bind(addr)
+    def sr(self, *args, **kargs):
+        return sendrecv.sndrcv(self, *args, **kargs)
+    def sr1(self, *args, **kargs):        
+        a,b = sendrecv.sndrcv(self, *args, **kargs)
+        if len(a) > 0:
+            return a[0][1]
+        else:
+            return None
+    def sniff(self, *args, **kargs):
+        return sendrecv.sniff(opened_socket=self, *args, **kargs)
 
 class L3RawSocket(SuperSocket):
     desc = "Layer 3 using Raw sockets (PF_INET/SOCK_RAW)"
@@ -51,8 +62,38 @@ class L3RawSocket(SuperSocket):
         self.outs = socket.socket(socket.AF_INET, socket.SOCK_RAW, socket.IPPROTO_RAW)
         self.outs.setsockopt(socket.SOL_IP, socket.IP_HDRINCL, 1)
         self.ins = socket.socket(socket.AF_PACKET, socket.SOCK_RAW, socket.htons(type))
-    def recv(self, x):
-        return Ether(self.ins.recv(x)).payload
+        if iface is not None:
+            self.ins.bind((iface, type))
+    def recv(self, x=MTU):
+        pkt, sa_ll = self.ins.recvfrom(x)
+        if sa_ll[2] == socket.PACKET_OUTGOING:
+            return None
+        if sa_ll[3] in conf.l2types:
+            cls = conf.l2types[sa_ll[3]]
+            lvl = 2
+        elif sa_ll[1] in conf.l3types:
+            cls = conf.l3types[sa_ll[1]]
+            lvl = 3
+        else:
+            cls = conf.default_l2
+            warning("Unable to guess type (interface=%s protocol=%#x family=%i). Using %s" % (sa_ll[0],sa_ll[1],sa_ll[3],cls.name))
+            lvl = 3
+
+        try:
+            pkt = cls(pkt)
+        except KeyboardInterrupt:
+            raise
+        except:
+            if conf.debug_dissector:
+                raise
+            pkt = conf.raw_layer(pkt)
+        if lvl == 2:
+            pkt = pkt.payload
+            
+        if pkt is not None:
+            from arch import get_last_packet_timestamp
+            pkt.time = get_last_packet_timestamp(self.ins)
+        return pkt
     def send(self, x):
         try:
             sx = str(x)
@@ -82,7 +123,7 @@ class StreamSocket(SimpleSocket):
         if x == 0:
             raise socket.error((100,"Underlying stream socket tore down"))
         pkt = self.basecls(pkt)
-        pad = pkt[Padding]
+        pad = pkt.getlayer(Padding)
         if pad is not None and pad.underlayer is not None:
             del(pad.underlayer.payload)
         while pad is not None and not isinstance(pad, NoPayload):
@@ -95,3 +136,5 @@ class StreamSocket(SimpleSocket):
 
 if conf.L3socket is None:
     conf.L3socket = L3RawSocket
+
+import sendrecv
