@@ -3,20 +3,36 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+General utility functions.
+"""
+
 import os,sys,socket,types
-from config import conf
-import random
+import random,time
 import gzip,zlib,cPickle
 import re,struct,array
-from arch import *
-from error import log_runtime
+import subprocess
+
+import warnings
+warnings.filterwarnings("ignore","tempnam",RuntimeWarning, __name__)
+
+from config import conf
+from data import MTU
+from error import log_runtime,log_loading,log_interactive
 from base_classes import BasePacketList
 
+WINDOWS=sys.platform.startswith("win32")
 
 ###########
 ## Tools ##
 ###########
 
+def get_temp_file(keep=False, autoext=""):
+    f = os.tempnam("","scapy")
+    if not keep:
+        conf.temp_files.append(f+autoext)
+    return f
+
 def sane_color(x):
     r=""
     for i in x:
@@ -202,7 +218,7 @@ def hexdiff(x,y):
     
 crc32 = zlib.crc32
 
-if BIG_ENDIAN:
+if struct.pack("H",1) == "\x00\x01": # big endian
     def checksum(pkt):
         if len(pkt) % 2 == 1:
             pkt += "\0"
@@ -233,6 +249,27 @@ def str2mac(s):
 def strxor(x,y):
     return "".join(map(lambda x,y:chr(ord(x)^ord(y)),x,y))
 
+# Workarround bug 643005 : https://sourceforge.net/tracker/?func=detail&atid=105470&aid=643005&group_id=5470
+try:
+    socket.inet_aton("255.255.255.255")
+except socket.error:
+    def inet_aton(x):
+        if x == "255.255.255.255":
+            return "\xff"*4
+        else:
+            return socket.inet_aton(x)
+else:
+    inet_aton = socket.inet_aton
+
+inet_ntoa = socket.inet_ntoa
+try:
+    inet_ntop = socket.inet_ntop
+    inet_pton = socket.inet_pton
+except AttributeError:
+    from scapy.pton_ntop import *
+    log_loading.info("inet_ntop/pton functions not found. Python IPv6 support not present")
+
+
 def atol(x):
     try:
         ip = inet_aton(x)
@@ -240,12 +277,12 @@ def atol(x):
         ip = inet_aton(socket.gethostbyname(x))
     return struct.unpack("!I", ip)[0]
 def ltoa(x):
-    return inet_ntoa(struct.pack("!I", x))
+    return inet_ntoa(struct.pack("!I", x&0xffffffff))
 
 def itom(x):
     return (0xffffffff00000000L>>x)&0xffffffffL
 
-def do_graph(graph,prog=None,format="svg",target=None, type=None,string=None,options=None):
+def do_graph(graph,prog=None,format=None,target=None,type=None,string=None,options=None):
     """do_graph(graph, prog=conf.prog.dot, format="svg",
          target="| conf.prog.display", options=None, [string=1]):
     string: if not None, simply return the graph string
@@ -255,20 +292,43 @@ def do_graph(graph,prog=None,format="svg",target=None, type=None,string=None,opt
     prog: which graphviz program to use
     options: options to be passed to prog"""
         
-
+    if format is None:
+        if WINDOWS:
+            format = "png" # use common format to make sure a viewer is installed
+        else:
+            format = "svg"
     if string:
         return graph
     if type is not None:
         format=type
     if prog is None:
         prog = conf.prog.dot
+    start_viewer=False
     if target is None:
-        target = "| %s" % conf.prog.display
+        if WINDOWS:
+            tempfile = os.tempnam("", "scapy") + "." + format
+            target = "> %s" % tempfile
+            start_viewer = True
+        else:
+            target = "| %s" % conf.prog.display
     if format is not None:
         format = "-T %s" % format
     w,r = os.popen2("%s %s %s %s" % (prog,options or "", format or "", target))
     w.write(graph)
     w.close()
+    if start_viewer:
+        # Workaround for file not found error: We wait until tempfile is written.
+        waiting_start = time.time()
+        while not os.path.exists(tempfile):
+            time.sleep(0.1)
+            if time.time() - waiting_start > 3:
+                warning("Temporary file '%s' could not be written. Graphic will not be displayed." % tempfile)
+                break
+        else:  
+            if conf.prog.display == conf.prog._default:
+                os.startfile(tempfile)
+            else:
+                subprocess.Popen([conf.prog.display, tempfile])
 
 _TEX_TR = {
     "{":"{\\tt\\char123}",
@@ -352,48 +412,9 @@ class Enum_metaclass(type):
 
 
 
-
-##############################
-## Session saving/restoring ##
-##############################
-
-
-def save_session(fname, session=None, pickleProto=-1):
-    if session is None:
-        session = scapy_session
-
-    to_be_saved = session.copy()
-        
-    if to_be_saved.has_key("__builtins__"):
-        del(to_be_saved["__builtins__"])
-
-    for k in to_be_saved.keys():
-        if type(to_be_saved[k]) in [types.TypeType, types.ClassType, types.ModuleType]:
-             log_interactive.error("[%s] (%s) can't be saved." % (k, type(to_be_saved[k])))
-             del(to_be_saved[k])
-
-    try:
-        os.rename(fname, fname+".bak")
-    except OSError:
-        pass
-    f=gzip.open(fname,"wb")
-    cPickle.dump(to_be_saved, f, pickleProto)
-    f.close()
-
-def load_session(fname):
-    try:
-        s = cPickle.load(gzip.open(fname,"rb"))
-    except IOError:
-        s = cPickle.load(open(fname,"rb"))
-    scapy_session.clear()
-    scapy_session.update(s)
-
-def update_session(fname):
-    try:
-        s = cPickle.load(gzip.open(fname,"rb"))
-    except IOError:
-        s = cPickle.load(open(fname,"rb"))
-    scapy_session.update(s)
+###################
+## Object saving ##
+###################
 
 
 def export_object(obj):
@@ -419,7 +440,7 @@ def corrupt_bytes(s, p=0.01, n=None):
     if n is None:
         n = max(1,int(l*p))
     for i in random.sample(xrange(l), n):
-        s[i] = random.randint(0,255)
+        s[i] = (s[i]+random.randint(1,255))%256
     return s.tostring()
 
 @conf.commands.register
@@ -472,10 +493,10 @@ class RawPcapReader:
         elif  magic == "\xd4\xc3\xb2\xa1": #little endian
             self.endian = "<"
         else:
-            raise RuntimeWarning, "Not a pcap capture file (bad magic)"
+            raise Scapy_Exception("Not a pcap capture file (bad magic)")
         hdr = self.f.read(20)
         if len(hdr)<20:
-            raise RuntimeWarning, "Invalid pcap file (too short)"
+            raise Scapy_Exception("Invalid pcap file (too short)")
         vermaj,vermin,tz,sig,snaplen,linktype = struct.unpack(self.endian+"HHIIII",hdr)
 
         self.linktype = linktype
@@ -493,7 +514,7 @@ class RawPcapReader:
         return pkt
 
 
-    def read_packet(self):
+    def read_packet(self, size=MTU):
         """return a single packet read from the file
         
         returns None when no more packets are available
@@ -502,7 +523,7 @@ class RawPcapReader:
         if len(hdr) < 16:
             return None
         sec,usec,caplen,wirelen = struct.unpack(self.endian+"IIII", hdr)
-        s = self.f.read(caplen)
+        s = self.f.read(caplen)[:MTU]
         return s,(sec,usec,wirelen) # caplen = len(s)
 
 
@@ -528,10 +549,10 @@ class RawPcapReader:
             res.append(p)
         return res
 
-    def recv(self, size):
+    def recv(self, size=MTU):
         """ Emulate a socket
         """
-        return self.read_packet()[0]
+        return self.read_packet(size)[0]
 
     def fileno(self):
         return self.f.fileno()
@@ -548,9 +569,9 @@ class PcapReader(RawPcapReader):
             self.LLcls = conf.l2types[self.linktype]
         except KeyError:
             warning("PcapReader: unknown LL type [%i]/[%#x]. Using Raw packets" % (self.linktype,self.linktype))
-            self.LLcls = Raw
-    def read_packet(self):
-        rp = RawPcapReader.read_packet(self)
+            self.LLcls = conf.raw_layer
+    def read_packet(self, size=MTU):
+        rp = RawPcapReader.read_packet(self,size)
         if rp is None:
             return None
         s,(sec,usec,wirelen) = rp
@@ -562,14 +583,15 @@ class PcapReader(RawPcapReader):
         except:
             if conf.debug_dissector:
                 raise
-            p = Raw(s)
+            p = conf.raw_layer(s)
         p.time = sec+0.000001*usec
         return p
     def read_all(self,count=-1):
         res = RawPcapReader.read_all(self, count)
-        return PacketList(res,name = os.path.basename(self.filename))
-    def recv(self, size):
-        return self.read_packet()
+        import plist
+        return plist.PacketList(res,name = os.path.basename(self.filename))
+    def recv(self, size=MTU):
+        return self.read_packet(size)
         
 
 
@@ -643,7 +665,7 @@ class RawPcapWriter:
             if sec is None:
                 sec = it
             if usec is None:
-                usec = int((t-it)*1000000)
+                usec = int(round((t-it)*1000000))
         self.f.write(struct.pack(self.endian+"IIII", sec, usec, caplen, wirelen))
         self.f.write(packet)
         if self.gz and self.sync:
@@ -668,13 +690,13 @@ class PcapWriter(RawPcapWriter):
 
     def _write_packet(self, packet):        
         sec = int(packet.time)
-        usec = int((packet.time-sec)*1000000)
+        usec = int(round((packet.time-sec)*1000000))
         s = str(packet)
         caplen = len(s)
         RawPcapWriter._write_packet(self, s, sec, usec, caplen, caplen)
 
 
-re_extract_hexcap = re.compile("^(0x[0-9a-fA-F]{2,}[ :\t]|(0x)?[0-9a-fA-F]{2,}:|(0x)?[0-9a-fA-F]{3,}[: \t]|) *(([0-9a-fA-F]{2} {,2}){,16})")
+re_extract_hexcap = re.compile("^((0x)?[0-9a-fA-F]{2,}[ :\t]{,3}|) *(([0-9a-fA-F]{2} {,2}){,16})")
 
 def import_hexcap():
     p = ""
@@ -682,7 +704,7 @@ def import_hexcap():
         while 1:
             l = raw_input().strip()
             try:
-                p += re_extract_hexcap.match(l).groups()[3]
+                p += re_extract_hexcap.match(l).groups()[2]
             except:
                 warning("Parsing error during hexcap")
                 continue
@@ -690,26 +712,23 @@ def import_hexcap():
         pass
     
     p = p.replace(" ","")
-    p2=""
-    for i in range(len(p)/2):
-        p2 += chr(int(p[2*i:2*i+2],16))
-    return p2
+    return p.decode("hex")
         
 
 
 @conf.commands.register
 def wireshark(pktlist):
     """Run wireshark on a list of packets"""
-    f = os.tempnam("scapy")
+    f = get_temp_file()
     wrpcap(f, pktlist)
-    os.spawnlp(os.P_NOWAIT, conf.prog.wireshark, conf.prog.wireshark, "-r", f)
+    subprocess.Popen([conf.prog.wireshark, "-r", f])
 
 @conf.commands.register
 def hexedit(x):
     x = str(x)
-    f = os.tempnam("scapy")
+    f = get_temp_file()
     open(f,"w").write(x)
-    os.spawnlp(os.P_WAIT, conf.prog.hexedit, conf.prog.hexedit, f)
+    subprocess.call([conf.prog.hexedit, f])
     x = open(f).read()
     os.unlink(f)
     return x
@@ -783,7 +802,3 @@ def make_lined_table(*args, **kargs):
 def make_tex_table(*args, **kargs):
     __make_table(lambda l: "%s", lambda l: "& %s", "\\\\", seplinefunc=lambda a,x:"\\hline", *args, **kargs)
 
-
-def tethereal(*args,**kargs):
-    sniff(prn=lambda x: x.display(),*args,**kargs)
-
